@@ -163,7 +163,8 @@ class PostScriptDevice(CommonDevice):
 		write("%%EndComments\n\n")
 
 		write('%%BeginProlog\n')
-		procfile = os.path.join(config.sketch_dir, config.postscript_prolog)
+		#procfile = os.path.join(config.user_config_dir, config.postscript_prolog)
+		procfile=config.postscript_prolog
 		procfile = open(procfile, 'r')
 		line = procfile.readline()
 		self.supplied_resources.append(join(split(strip(line))[1:]))
@@ -295,7 +296,6 @@ class PostScriptDevice(CommonDevice):
 
 	def _set_color(self, color):
 		if self.current_color != color:
-			print color
 			if color.model == 'CMYK':
 				c, m, y, k = color.getCMYK()
 				self.file.write('%g %g %g %g cmyk\n' % (round(c, 3), round(m, 3), round(y, 3), round(k, 3)))
